// This script contains a number of useful procedures for printing text on the screen.
(version 2)
(include "sci.sh")
(use "Main")
(use "Controls")
(use "DialogControls")
(use "Obj")
(script 921)


/*
	The Print class lets you show text, buttons and images in a GUI on screen.
	
	It is generally used as a static class, in the sense that you call methods on Print
	directly without making an instance of it. The init() method then displays the dialog.
	
	Example usage::
		
		(Print:
			addTitle("This is the title")
			font(0)
			width(90)
			addText("Enter new font number:")
			addEdit(@temp0 6 0 24)
			addButton(0 "A button!" 0 12)
			init()
		)

*/
(class Print of Obj
    (properties
        dialog 0
        window 0
        title 0
        mode 0
        font -1
        width 0
        x -1
        y -1
        ticks 0
        caller 0
        retValue 0
        modeless 0
        first 0
        saveCursor 0
    )

	/*
	.. function:: init([theCaller text])
	
		Calls showSelf() to show the dialog.
	
		:param heapPtr theCaller: An optional object with a cue method.
		:param string text: Optional text that is added to the dialog.
		:returns: If a button was pressed, returns the value of that button.
	*/
    (method (init theCaller theText)
        = caller 0
        (if (paramTotal)
            = caller theCaller
        )
        (if (> paramTotal 1)
            (self:addText(rest theText))
        )
        (if (not modeless)
            (if (not IsObject(gPrintEventHandler))
                = gPrintEventHandler (send ((EventHandler:new())):name("prints"))
            )
            (send gPrintEventHandler:add(self))
        )
        (self:showSelf())
    )

    (method (doit)
        (send dialog:eachElementDo(#doit))
    )

    (method (dispose)
        (if (gPrintEventHandler and (send gPrintEventHandler:contains(self)))
            (send gPrintEventHandler:delete(self))
            (if ((send gPrintEventHandler:isEmpty()))
                (send gPrintEventHandler:dispose())
                = gPrintEventHandler NULL
            )
        )
        (if (title)
            Memory(memFREE title)
        )
        = width (= mode (= title (= first (= saveCursor (= window 0)))))
        = x (= y -1)
        = modeless 0
        (send gSounds:pause(0))
        (super:dispose())
    )

	// Shows the dialog. Generally, you should use the init() method to show the dialog.
    (method (showSelf)
        (var theFirst, temp1, temp2, temp3, temp4)
        (if (saveCursor)
            (send gGame:setCursor(999))
        )
        (if (not dialog)
            = dialog (Dialog:new())
        )
        (send dialog:
            window(
                (if (window)
                )(else
                    gWindow
                )
            )
            name("PODialog")
            caller(self)
        )
        (send dialog:
            text(title)
            time(ticks)
            setSize()
        )
        (send dialog:center())
        = temp3 
            (if (== x -1)
                (send dialog:nsLeft)
            )(else
                x
            )
        = temp4 
            (if (== y -1)
                (send dialog:nsTop)
            )(else
                y
            )
        (send dialog:moveTo(temp3 temp4))
        = temp1 GetPort()
        (send dialog:open( (if (title) nwTITLE)(else nwNORMAL) 15))
        return 
            (if (modeless)
                = gOldPort GetPort()
                SetPort(temp1)
                = gDialog dialog
            )(else
                (send gSounds:pause(1))
                = theFirst first
                (if (not theFirst)
                    = theFirst (send dialog:firstTrue(#checkState 1))
                    (if (theFirst and not (send dialog:firstTrue(#checkState 2)))
                        (send theFirst:state((| (send theFirst:state) $0002)))
                    )
                )(else
                    (if (not IsObject(theFirst))
                        = theFirst (send dialog:at(theFirst))
                    )
                )
                = retValue (send dialog:doit(theFirst))
                SetPort(temp1)
                (if (== retValue -1)
                    = retValue 0
                )(else
                    (if (IsObject(retValue) and (send retValue:isKindOf(DButton)))
                        = retValue (send retValue:value)
                    )(else
                        (if (not (send dialog:theItem))
                            = retValue 1
                        )
                    )
                )
                (send dialog:dispose())
                return retValue
            )
    )

	/*
	.. function:: addButton(theValue noun verb cond seq [x y modNum])
	
	.. function:: addButton(theValue text [x y])
	
		Adds a button to the dialog, either with the supplied text or a message resource. 
	
		:param number theValue: A numerical value associated with the button. This value will be returned by the dialog if the button is pressed.
		:param string text: The button text.
		:param number x: The button x offset from the upper left of the dialog.
		:param number y: The button y offset from the upper left of the dialog.
		:param number noun: The noun of the message to show.
		:param number verb: The verb of the message to show.
		:param number cond: The condition of the message to show.
		:param number seq: The sequence of the message to show, or 0.
		:param number modNum: Optional room number of the message to show (if not specified, the current room is used).
	*/
    (method (addButton theValue params)
        (var noun, verb, cond, seq, x, y, modNum, text, temp8)
        (if (not dialog)
            = dialog (Dialog:new())
        )
        (if (== font -1)
            = font gFont
        )
        = x 0
        = y 0
        (if (> paramTotal 2)
            = x params[1]
            (if (> paramTotal 3)
                = y params[2]
            )
        )
        = text Memory(memALLOC_CRIT (+ StrLen(params[0]) 1))
        StrCpy(text params[0])
        (if (text)
            (send dialog:
                add((send ((DButton:new())):
                        value(theValue)
                        font(font)
                        text(text)
                        setSize()
                        moveTo((+ 4 x) (+ 4 y))
                        yourself()
                    )
                     )
                setSize()
            )
        )
    )

	/*
	.. function:: addText(noun verb cond seq [x y modNum])
	
	.. function:: addText(text [x y])
	
		Adds text to the dialog, either with the supplied string or a message resource. 
	
		:param string text: The text.
		:param number x: The text x offset from the upper left of the dialog.
		:param number y: The text y offset from the upper left of the dialog.
		:param number noun: The noun of the message to show.
		:param number verb: The verb of the message to show.
		:param number cond: The condition of the message to show.
		:param number seq: The sequence of the message to show, or 0.
		:param number modNum: Optional room number of the message to show (if not specified, the current room is used).
	*/    
    (method (addText params)
        (var noun, verb, cond, seq, x, y, modNum, text, temp8)
        (if (not dialog)
            = dialog (Dialog:new())
        )
        (if (== font -1)
            = font gFont
        )
        = x 0
        = y 0
        (if (>= paramTotal 2)
            = x params[1]
            (if (>= paramTotal 3)
                = y params[2]
            )
        )
        = text Memory(memALLOC_CRIT (+ StrLen(params[0]) 1))
        StrCpy(text params[0])
        (send dialog:
            add((send ((DText:new())):
                    text(text)
                    font(font)
                    mode(mode)
                    setSize(width)
                    moveTo(+ 4 x + 4 y)
                    yourself()
                )
            )
            setSize()
        )
    )

	/*
	.. function:: addTitle(noun verb cond seq [modNum])
	
	.. function:: addTitle(text)
	
		Adds a title to the dialog, either with the supplied string or a message resource. 
	
		:param string text: The title text.
		:param number noun: The noun of the message to show.
		:param number verb: The verb of the message to show.
		:param number cond: The condition of the message to show.
		:param number seq: The sequence of the message to show, or 0.
		:param number modNum: Optional room number of the message to show (if not specified, the current room is used).
	*/
    (method (addTitle params)
        (var noun, verb, cond, seq, modNum, temp5)
        = title Memory(memALLOC_CRIT (+ StrLen(params[0]) 1))
        StrCpy(title params[0])
    )

	// Positions the dialog on screen.
    (method (posn theX theY)
        = x theX
        = y theY
    )

    (method (handleEvent pEvent)
        (if ((send dialog:handleEvent(pEvent)))
            (send dialog:dispose())
        )
    )

    (method (cue)
        (var theCaller)
        = theCaller caller
        = dialog 0
        (if (window)
            (send window:dispose())
        )
        (self:dispose())
        (if (theCaller)
            (send theCaller:cue())
        )
    )
)
